package org.dromara.system.service;

import org.dromara.system.domain.vo.TblEventVo;
import org.dromara.system.domain.bo.TblEventBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 事件Service接口
 *
 * @author Lion Li
 * @date 2025-06-13
 */
public interface ITblEventService {

    /**
     * 查询事件
     *
     * @param id 主键
     * @return 事件
     */
    TblEventVo queryById(Long id);

    /**
     * 分页查询事件列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 事件分页列表
     */
    TableDataInfo<TblEventVo> queryPageList(TblEventBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的事件列表
     *
     * @param bo 查询条件
     * @return 事件列表
     */
    List<TblEventVo> queryList(TblEventBo bo);

    /**
     * 新增事件
     *
     * @param bo 事件
     * @return 是否新增成功
     */
    Boolean insertByBo(TblEventBo bo);

    /**
     * 修改事件
     *
     * @param bo 事件
     * @return 是否修改成功
     */
    Boolean updateByBo(TblEventBo bo);

    /**
     * 校验并批量删除事件信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
